{******************************************************************************}
{       odAbout Unit                                                           }
{                                                                              }
{       This unit defines the TodAboutBoxProperty editor and the corresponding }
{       form that is displayed when invoked.                                   }
{                                                                              }
{       Copyright  2005 by OD                                                 }
{******************************************************************************}

unit odAbout;

interface

uses
  Messages, Windows, Classes, Graphics, Forms, Controls,
  DesignIntf, DesignEditors, StdCtrls, ExtCtrls, Buttons,
  odResStr;

type
  TodCompMap = record
                 Name: string;
                 Index: Word;
               end;

  TodAboutBoxProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;


  TodAboutBox = class(TForm)
    Panel3: TPanel;
    Image1: TImage;
    Label2: TLabel;
    lblEmail: TLabel;
    Label1: TLabel;
    lblVersion: TLabel;
    lblCopyright: TLabel;
    imgComp: TImage;
    LblComponentName: TLabel;
    Bevel1: TBevel;
    lblDescription: TLabel;
    btnClose: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure lblEmailMouseEnter(Sender: TObject);
    procedure lblEmailMouseLeave(Sender: TObject);
    procedure lblEmailClick(Sender: TObject);
  private
    function FindDescription(CompName: String): String;
  public
    procedure SetComponentName(Value: String);
  end;

const
  CompCount = 1;
  ResStringTable: array[1 .. CompCount] of TodCompMap =
                  (
                   (Name: 'TodMacOSCaption'; Index: T_odMacOSCaption)
                  );


implementation

{$R *.DFM}

{$R odResStr.res}

uses
  SysUtils, ShellAPI;

{******************************************************************************}
{               TodAboutBoxProperty                                            }
{******************************************************************************}
function TodAboutBoxProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TodAboutBoxProperty.GetValue: string;
begin
  Result := LoadStr(SProductVersion);
end;

procedure TodAboutBoxProperty.Edit;
var
  AboutDlg: TodAboutBox;
begin
  AboutDlg := TodAboutBox.Create(Application);
  try
    AboutDlg.SetComponentName(GetComponent(0).ClassName);
    AboutDlg.ShowModal;
  finally
    AboutDlg.Free;
  end;
end;
{******************************************************************************}
{               TodAboutBox                                                    }
{******************************************************************************}
procedure TodAboutBox.FormCreate(Sender: TObject);
begin
  lblCopyright.Caption := 'Copyright  ' + LoadStr(SCopyrightDate) + ' ' + LoadStr(SCompanyName);
  lblVersion.Caption := LoadStr(SProductVersion);
end;


function TodAboutBox.FindDescription(CompName: String): String;
var
  I: Integer;
begin
  Result := '';
  I := 1;
  while (I <= CompCount) and
        (CompareText(CompName, ResStringTable[I].Name) <> 0 ) do
  begin
    Inc(I);
  end;

  if I <= CompCount then
    Result := LoadStr(ResStringTable[I].Index)
end;


procedure TodAboutBox.SetComponentName(Value: String);
var
  BmpName: array[ 0..128 ] of Char;
begin
  if Value[1] = 'T' then
    lblComponentName.Caption := Copy(Value, 2, 255)
  else
    lblComponentName.Caption := Value;
//
  lblDescription.Caption := FindDescription(Value);

// Load Component Bitmap from Delphi's Component Library
  StrPCopy(BmpName, UpperCase(Value));
  imgComp.Picture.Bitmap.Handle := LoadBitmap(HInstance, BmpName);
  imgComp.Visible := True;
  imgComp.Transparent := True;
end;

procedure TodAboutBox.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TodAboutBox.lblEmailMouseEnter(Sender: TObject);
begin
  Screen.Cursor := crHandPoint;
end;

procedure TodAboutBox.lblEmailMouseLeave(Sender: TObject);
begin
  Screen.Cursor := crDefault;
end;

procedure TodAboutBox.lblEmailClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', PChar('mailto:' + lblEmail.Caption), nil, nil, SW_SHOW);
end;

end.




