{******************************************************************************}
{       odMacOS Unit                                                           }
{                                                                              }
{       This unit defines TodCustomMacOS Component.                            }
{       Draw form caption in MacOS form style                                  }
{                                                                              }
{       Copyright  2005 by OD                                                 }
{                                                                              }
{                                                                              }
{       There are some problems while using this component.                    }
{       1. If form have MainMenu!!!                                            }
{          The problem is that MainMenu is not painting.                       }
{          I solve this problem by putting on the form a TToolBar,             }
{          and assign MainMenu of the form to TToolBar, and clear Menu         }
{          property of the parent form.                                        }
{          This method is not very good, because the client area of the form in}
{          case of such assignment is larger than if parent form have Menu     }
{          property set, in other words, if you will activate and deactivate   }
{          this component at runtime, you will see that all your component     }
{          on parent form goes up and down :))).                               }
{                                                                              }
{       2. It does not work properly if form is MDIForm or MDIChild            }
{          In this case there is some bugs, such as:                           }
{             - on MDIChild form you cannot click any buttons from caption     }
{             - When MDIChild is maximized, its caption disappeare, and you    }
{               cannot restore, close or minimize form.                        }
{             - When minimized, MDIChild form goes down, and you cannot        }
{               restore, close, maximized.                                     }
{             - I cannot retrieve form system menu.                           }
{                                                                              }
{       If you know how it can be correct please inform me.                    }
{                                                                              }
{       And if you find any bugs, also please inform me.                       }
{                                                                              }
{       And the last, you are freely to use and modify this component,         }
{       but a have a favour to ask of you, to inform me about modification     }
{       of this component.                                                     }
{                                                                              }
{       E-mail: odnics@gmail.com                                               }
{                                                                              }
{******************************************************************************}

unit odMacOS;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, Forms, ComCtrls,
  ToolWin, Dialogs, Types,
  odAbout, odCommon;

const
{******************************************************************************}
{                               New messages                                   }
{******************************************************************************}
// Set new message for Maximize and Close buttons
  WM_MAXHIT = WM_USER + 1;
  WM_CLOSEHIT = WM_USER + 2;
{******************************************************************************}
{                               Set defaults                                   }
{******************************************************************************}
// Set default spaces
  _BCSpace = 3;            // Space between form caption and buttons
  _ButtonSpace = 2;        // Space between buttons
  _ButtonEdgeSpace = 2;    // Space between form edge and buttons
// Set default sizes
  _ButtonSize = 14;        // Button size
  _FormEdgeSize = 2;       // Edge size
// Set default placement
  _CaptionTop = 5;         // Distance to form caption
  _CaptionFontTop = 5;     // Distance to form caption text
// Set default font
  DefaultFont = 'MS Sans Serif';  // Caption font
  DefaultFontStyle = [fsBold];    // Caption font style
  DefaultFontSize = 8;            // Caption font size

type
  TodCustomMacOSCaption = class(TComponent)
  private
    FPrevWndProc	: Pointer;
    FNewWndProc 	: Pointer;
//
    FAbout              : TodAbout;
//
    FMacFormActive      : Boolean;
    FMacCapStyle        : TodMacCapStyle;               // Caption style
    FColorSchema        : TodMacColorSchema;            // Color scheme
    FCloseRgn, FMinRgn,
    FMaxRgn, FIconRgn   : TRect;                        // Buttons and icon effect region
    FBtnSet             : TodButtonSets;                // Buttons set
    FBtnState           : array[1..3] of TodButtonState;// Buttons state

    FActCapColor,                                       // Active caption color
    FInactCapColor      : TColor;                       // Inactive caption color
    FActFontColor,                                      // Active caption text color
    FInactFontColor     : TColor;                       // Inactive caption text color

    FCapFont            : TFont;                        // Caption font
    FLDColor            : TColor;                       // Line/point color in caption (for caption style = mfsMacOSLine, mfsMacOSDot)
    FLDShadow           : TColor;                       // Line/point shadow color in caption (for caption style = mfsMacOSLine, mfsMacOSDot)
    FBorderColor        : TColor;                       // Border color
    FPaintBorder        : Boolean;                      // Paint or not border

    FCaptionBmp: TBitMap;                               // Caption bitmap
    FCapButton: TBitmap;                                // button image
//    FMenuOffset:Integer;                                // Main menu offset

    CY_CAPTION		: Integer;
    CX_ICON		: Integer;
    CY_ICON		: Integer;
    CX_EDGE             : Integer;
    CY_EDGE             : Integer;

// Creating caption procedures
    procedure CreateCaption(Rec: TRect; Name: String; CActive: Boolean; var BMP: TBitMap);
    procedure DrawCap(AActive: Boolean; ClBtn, MaxBtn, MinBtn: String);
    procedure DrawIcon(DC: HDC; R: TRect);
    procedure PaintBorder;
// Set properties procedures
    procedure SetMacFormActive(Value: Boolean);
    procedure SetCapStyle(Value: TodMacCapStyle);
    procedure SetColorSchema(Value: TodMacColorSchema);
    procedure SetBorderIcons(Value: TodButtonSets);
    procedure SetActiveCapColor(Value: TColor);
    procedure SetInactCapColor(Value: TColor);
    procedure SetActFontColor(Value: TColor);
    procedure SetInactFontColor(Value: TColor);
    procedure SetCapFont(Value: TFont);
    procedure SetLDColor(Value: TColor);
    procedure SetLDShadow(Value: TColor);
    procedure SetBorderColor(Value: TColor);
    procedure SetPaintBorder(Value: Boolean);
// Special procedures
    procedure Repaint;
    procedure ParentWindowHookProc(var Msg_: TMessage);
    procedure SetParentWindowHook;
    procedure FreeParentWindowHook;
//
  protected
    procedure Notification( Component: TComponent; Operation: TOperation ); override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    function GetWindowButtons(FMacBtnSet: TodButtonSets): TBorderIcons;
  published
// Properties
    property About: TodAbout read FAbout write FAbout stored False;
    property Active: Boolean read FMacFormActive write SetMacFormActive
      default False;
    property CapStyle: TodMacCapStyle read FMacCapStyle write SetCapStyle
      default mcsMacOSLine;
    property ColorSchema: TodMacColorSchema read FColorSchema write SetColorSchema;
    property ActiveCapColor: TColor read FActCapColor write SetActiveCapColor
      default clCaptionText;
    property InactiveCapColor: TColor read FInactCapColor write SetInactCapColor
      default clInactiveCaptionText;
    property ActiveFontColor: TColor read FActFontColor write SetActFontColor
      default clActiveCaption;
    property InactiveFontColor: TColor read FInactFontColor write SetInactFontColor
      default clInactiveCaption;
    property CaptionFont: TFont read FCapFont write SetCapFont;
    property LDColor: TColor read FLDColor write SetLDColor
      default clWhite;
    property LDShadow: TColor read FLDShadow write SetLDShadow
      default clBtnShadow;
    property BorderColor: TColor read FBorderColor write SetBorderColor
      default clBtnShadow;
    property PaintFormBorder: Boolean read FPaintBorder write SetPaintBorder
      default True;
    property ButtonSets: TodButtonSets read FBtnSet write SetBorderIcons;
  end;

  TodMacOSCaption = class(TodCustomMacOSCaption)
  published
    property About;
    property Active;
    property CapStyle;
    property ColorSchema;
    property ActiveCapColor;
    property InactiveCapColor;
    property ActiveFontColor;
    property InactiveFontColor;
    property CaptionFont;
    property LDColor;
    property LDShadow;
    property BorderColor;
    property PaintFormBorder;
    property ButtonSets;
  end;

var
  ODtlbMain: TToolBar;

implementation

{$R MacButton.res}

{******************************************************************************}
{               TodCustomMacOSCaption                                          }
{******************************************************************************}
{               Set properties procedures                                      }
{******************************************************************************}
{       TodCustomMacOSCaption.SetMacFormActive:                                }
{               Activate MacOS style caption                                   }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetMacFormActive(Value: Boolean);
begin
  FMacFormActive := Value;
  if FMacFormActive then
  begin
    if not (csDesigning in ComponentState) then
    begin
      if Assigned(TForm(Owner).Menu) then
      begin
        if not Assigned(ODtlbMain) then
          ODtlbMain := TToolBar.Create(Owner);
        with ODtlbMain do
        begin
          Parent := TForm(Owner);
          Flat := True;
          Align := alTop;
          EdgeBorders := [];
          AutoSize := True;
          Menu := TForm(Owner).Menu;
          TForm(Owner).Menu := nil;
        end;
      end;
    end;
    TForm(Owner).BorderIcons := [];
    SetParentWindowHook;
  end
  else
  begin
//
    if Assigned(ODtlbMain) then
    begin
      TForm(Owner).Menu := ODtlbMain.Menu;
      ODtlbMain.Menu := nil;
      FreeAndNil(ODtlbMain);
    end;  
//
    FreeParentWindowHook;
    TForm(Owner).BorderIcons := GetWindowButtons(FBtnSet);
  end;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetCapStyle:                                     }
{               Set MacOS form style                                           }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetCapStyle(Value: TodMacCapStyle);
begin
  FMacCapStyle := Value;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetColorSchema:                                  }
{               Set MacOS color scheme (for MacOS form style mfsColor )        }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetColorSchema(Value: TodMacColorSchema);
begin
  if CapStyle = mcsColor then
  begin
    if Value <> FColorSchema then
    begin
      FColorSchema := Value;
      case FColorSchema of
        mcsWindows: begin
                     ActiveCapColor := clActiveCaption;
                     InactiveCapColor := clInactiveCaption;
                     ActiveFontColor := clCaptionText;
                     InactiveFontColor := clBtnShadow;
                   end;
        mcsYellow: begin
                    ActiveCapColor := clInfoBk;
                    InactiveCapColor := clCream;
                    ActiveFontColor := clActiveCaption;
                    InactiveFontColor := clInactiveCaption;
                   end;
       mcsMarine: begin
                   ActiveCapColor := clNavy;
                   InactiveCapColor := clSkyBlue;
                   ActiveFontColor := clActiveCaption;
                   InactiveFontColor := clInactiveCaption;
                 end;

      end;
    end;
    Repaint;
  end;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetLDColor:                                      }
{               Set line/point color                                           }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetLDColor(Value: TColor);
begin
  FLDColor := Value;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetLDShadow:                                     }
{               Set line/point shadow color                                    }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetLDShadow(Value: TColor);
begin
  FLDShadow := Value;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetCapFont:                                      }
{               Set form caption text font                                     }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetCapFont(Value: TFont);
begin
  if not Assigned(Value) then
    exit;
  FCapFont.Assign(Value);
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetActiveCapColor:                               }
{               Set active caption color                                       }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetActiveCapColor(Value: TColor);
begin
  FActCapColor := Value;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetInactCapColor:                                }
{               Set inactive caption color                                     }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetInactCapColor(Value: TColor);
begin
  FInactCapColor := Value;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetActFontColor:                                 }
{               Set active caption font color                                  }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetActFontColor(Value: TColor);
begin
  FActFontColor := Value;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetInactFontColor:                               }
{               Set inactive caption font color                                }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetInactFontColor(Value: TColor);
begin
  FInactFontColor := Value;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetBorderIcons:                                  }
{               Set active caption buttons                                     }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetBorderIcons(Value: TodButtonSets);
begin
  if FBtnSet <> Value then
  begin
    FBtnSet := Value;
  end;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetBorderColor:                                  }
{               Set form border color                                          }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetBorderColor(Value: TColor);
begin
  FBorderColor := Value;
  Repaint;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetPaintBorder:                                  }
{               Set to paint or not form border                                }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetPaintBorder(Value: Boolean);
begin
  FPaintBorder := Value;
  Repaint;
end;

{******************************************************************************}
{               System message processing (Parent WndProc)                     } 
{******************************************************************************}
{       TodCustomMacOSCaption.ParentWindowHookProc:                            }
{               Processing system message for parent form                      }
{               (WndProc)                                                      }
{******************************************************************************}
procedure TodCustomMacOSCaption.ParentWindowHookProc(var Msg_: TMessage);
var
  pt: TPoint;
  OwnerFormCoord: TPoint;
  LItem : LongWord;
  LMenu : HMENU;
//
  procedure DefaultProc;
  begin
    with Msg_ do
      Result := CallWindowProc(FPrevWndProc, TForm(Owner).Handle, Msg, WParam, LParam);
  end;

begin
  if FMacFormActive then
  begin
    OwnerFormCoord.X := TForm(Owner).Left;
    OwnerFormCoord.Y := TForm(Owner).Top;
//
    with Msg_ do
    begin
      case Msg of
        WM_SIZE: begin
                   DefaultProc;
                   DrawCap(True, '', '', '');
                 end;
        WM_NCPAINT,
        WM_MOUSEACTIVATE:
                 begin
                   DefaultProc;
                   TForm(Owner).BorderIcons := [];
                   DrawCap(TForm(Owner).Active, '', '', '');
                 end;
        WM_NCACTIVATE: begin
                         DefaultProc;
                         DrawCap(TWMNCACTIVATE(Msg_).Active, '', '', '');
                       end;
        WM_NCHITTEST: begin
                        DefaultProc;
                        pt.X := TWMNCHITTEST(Msg_).XPos;
                        pt.Y := TWMNCHITTEST(Msg_).YPos;
                        if (not PtInRect(FMaxRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))
                              And(FBtnState[2].Pressed) then
                        begin
                          if FBtnState[2].Pressed then
                          begin
                            DrawCap(TForm(Owner).Active, '', '', '');
                            FBtnState[2].Pressed := False;
                          end
                        end;
                        if (not PtInRect(FCloseRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))
                              And(FBtnState[1].Pressed) then
                        begin
                          if FBtnState[1].Pressed then
                          begin
                            DrawCap(TForm(Owner).Active, '', '', '');
                            FBtnState[1].Pressed := False;
                          end
                        end;
                        if (not PtInRect(FMinRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))
                              And(FBtnState[3].Pressed) then
                        begin
                          if FBtnState[3].Pressed then
                          begin
                            DrawCap(TForm(Owner).Active, '', '', '');
                            FBtnState[3].Pressed := False;
                          end
                        end;
                      end;
        WM_NCLBUTTONDOWN: begin
                            pt.X := TWMNCLBUTTONDOWN(Msg_).XCursor;
                            pt.Y := TWMNCLBUTTONDOWN(Msg_).YCursor;
                            if (TWMNCLBUTTONDOWN(Msg_).HitTest = 2) then
                            begin
                              if(not PtInRect(FMaxRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))AND
                                (not PtInRect(FCloseRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))AND
                                (not PtInRect(FMinRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))AND
                                (not PtInRect(FIconRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y))) then
                                DefaultProc;
                              if btnMaximize in FBtnSet then
                                if PtInRect(FMaxRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)) then
                                begin
                                  DrawCap(TForm(Owner).Active, '', 'I', '');
                                  FBtnState[2].Pressed := True;
                                end;
                              if btnClose in FBtnSet then
                                if PtInRect(FCloseRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)) then
                                begin
                                  DrawCap(TForm(Owner).Active, 'I', '', '');
                                  FBtnState[1].Pressed := True;
                                end;
                              if btnMinimize in FBtnSet then
                                if PtInRect(FMinRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)) then
                                begin
                                  DrawCap(TForm(Owner).Active, '', '', 'I');
                                  FBtnState[3].Pressed := True;
                                end;
                              if PtInRect(FIconRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)) then
                              begin
                                LMenu := GetSystemMenu(Application.Handle, False);
                                LItem := LongWord(Windows.TrackPopupMenu(LMenu, TPM_LEFTBUTTON or TPM_TOPALIGN or
                                                                     TPM_RIGHTBUTTON or TPM_RETURNCMD,
                                                                     pt.X, OwnerFormCoord.Y + CY_CAPTION, 0, Application.Handle, nil));
                                if LItem > 0 then
                                  SendMessage(TForm(Owner).Handle, WM_SYSCOMMAND, LItem, 0);
                              end;
                            end
                            else
                              DefaultProc;
                          end;
        WM_NCLBUTTONUP: begin
                          pt.X := TWMNCLBUTTONUP(Msg_).XCursor;
                          pt.Y := TWMNCLBUTTONUP(Msg_).YCursor;
                          if TWMNCLBUTTONUP(Msg_).HitTest = 2 then
                          begin
                            if btnMaximize in FBtnSet then
                              if PtInRect(FMaxRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)) then
                              begin
                                DrawCap(TForm(Owner).Active, '', '', '');
                                FBtnState[2].Pressed := False;
                                SendMessage(TForm(Owner).Handle, WM_MAXHIT, 0, 0);
                              end;
                            if btnClose in FBtnSet then
                              if PtInRect(FCloseRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)) then
                              begin
                                DrawCap(TForm(Owner).Active, '', '', '');
                                FBtnState[1].Pressed := True;
                                SendMessage(TForm(Owner).Handle, WM_CLOSEHIT, 0, 0);
                              end;
                            if btnMinimize in FBtnSet then
                              if PtInRect(FMinRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)) then
                              begin
                                DrawCap(TForm(Owner).Active, '', '', '');
                                FBtnState[3].Pressed := True;
                                SendMessage(TForm(Owner).Handle, WM_SYSCOMMAND, SC_MINIMIZE, 0);
                              end;
                          end
                          else
                            DefaultProc;
                        end;
        WM_NCLBUTTONDBLCLK: begin
                              pt.X := TWMNCLBUTTONDBLCLK(Msg_).XCursor;
                              pt.Y := TWMNCLBUTTONDBLCLK(Msg_).YCursor;
                              if TWMNCLBUTTONDBLCLK(Msg_).HitTest = 2 then
                              begin
                                if (not PtInRect(FCloseRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))And
                                   (not PtInRect(FMaxRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))And
                                   (not PtInRect(FMinRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y)))And
                                   (not PtInRect(FIconRgn, Point(pt.X - OwnerFormCoord.X, pt.Y - OwnerFormCoord.Y))) then
                                  if btnMaximize in FBtnSet then
                                    SendMessage(TForm(Owner).Handle, WM_MAXHIT, 0, 0);
                              end;
                              TWMNCLBUTTONDBLCLK(Msg_).HitTest := 0;
                              DefaultProc;
                            end;
        WM_MAXHIT: begin
                     if TForm(Owner).WindowState = wsNormal then
                       SendMessage(TForm(Owner).Handle, WM_SYSCOMMAND, SC_MAXIMIZE, 0)
                     else
                       SendMessage(TForm(Owner).Handle, WM_SYSCOMMAND, SC_RESTORE, 0);
                   end;
        WM_CLOSEHIT: SendMessage(TForm(Owner).Handle, WM_SYSCOMMAND, SC_CLOSE, 0);
        WM_DESTROY: begin
                      FreeParentWindowHook;
                      DefaultProc;
                    end;
        else
          DefaultProc;
      end;
    end;
  end
  else
  begin
    TForm(Owner).BorderIcons := GetWindowButtons(FBtnSet);
    DefaultProc;
  end;
end;

{******************************************************************************}
{                       Special procedures                                     }
{******************************************************************************}
{       TodCustomMacOSCaption.Repaint:                                         }
{               Repaint form caption                                           }
{******************************************************************************}
procedure TodCustomMacOSCaption.Repaint;
var
  RGN: HRGN;
begin
  RGN := CreateRectRgn(0, 0, TForm(Owner).Width, CY_CAPTION);
  SendMessage(THandle(TForm(Owner).Handle), WM_NCPAINT, HRGN(RGN), 0);
  DeleteObject(RGN);
end;
{******************************************************************************}
{       TodCustomMacOSCaption.Notification:                                    }
{               Processing adding new  TodCustomMacOSCaption to the form       }
{******************************************************************************}
procedure TodCustomMacOSCaption.Notification(Component: TComponent; Operation: TOperation);
begin
  if (Component <> Self)and
     (Operation = opInsert)and
     (Component is TodCustomMacOSCaption) then
    raise
      Exception.Create('Cannot create more than one instance of TodCustomMacOSCaption component');
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetParentWindowHook:                             }
{               Replacing old parent form WndProc with new one                 }
{******************************************************************************}
procedure TodCustomMacOSCaption.SetParentWindowHook;
var
  P: Pointer;
begin
  P := Pointer(GetWindowLong(TForm(Owner).Handle, GWL_WNDPROC));
  if (P <> FNewWndProc) then
  begin
    FPrevWndProc := P;
    FNewWndProc := MakeObjectInstance(ParentWindowHookProc);
    SetWindowLong(TForm(Owner).Handle, GWL_WNDPROC, LongInt(FNewWndProc));
  end;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.SetParentWindowHook:                             }
{               Restore old WndProc of parent form                             }
{******************************************************************************}
procedure TodCustomMacOSCaption.FreeParentWindowHook;
var
  ExistsWndProc: Pointer;
begin
  if FNewWndProc <> nil then
  begin
    ExistsWndProc := Pointer(GetWindowLong(TForm(Owner).Handle, GWL_WNDPROC));
    if (FPrevWndProc <> nil)and(ExistsWndProc = FNewWndProc) then
    begin
      SetWindowLong(TForm(Owner).Handle, GWL_WNDPROC, LongInt(FPrevWndProc));
      FNewWndProc := nil;
    end;
  end;
end;
{******************************************************************************}
{       TodCustomMacOSCaption.GetWindowButtons:                                }
{               Restore standard window buttons                                }
{******************************************************************************}
function TodCustomMacOSCaption.GetWindowButtons(FMacBtnSet: TodButtonSets): TBorderIcons;
begin
  Result := [];
  if btnClose in FMacBtnSet then
    Result := Result + [biSystemMenu];
  if btnMinimize in FMacBtnSet then
    Result := Result + [biMinimize];
  if btnMaximize in FMacBtnSet then
    Result := Result + [biMaximize];
end;

{******************************************************************************}
{                       Constructor/destructor                                 }
{******************************************************************************}
{       TodCustomMacOSCaption.Create:                                          }
{               Component constructor                                          }
{******************************************************************************}
constructor TodCustomMacOSCaption.Create(Owner: TComponent);
begin
  inherited Create(Owner);
//
  if not(Owner is TForm) then exit;
//
  if not (csDesigning in ComponentState) then
  begin
    if Assigned(TForm(Owner).Menu) then
    begin
      ODtlbMain := TToolBar.Create(Owner);
      with ODtlbMain do
      begin
        Parent := TForm(Owner);
        Flat := True;
        Align := alTop;
        EdgeBorders := [];
        AutoSize := True;
        Menu := TForm(Owner).Menu;
        TForm(Owner).Menu := nil;
      end;
    end;
  end;
//
  FCaptionBmp := TBitmap.Create;
  FillChar(FBtnState, SizeOf(FBtnState),0);
// Set buttons
  if biMinimize in TForm(Owner).BorderIcons then
    FBtnSet := FBtnSet + [btnMinimize]
  else
    FBtnSet := FBtnSet - [btnMinimize];

  if biMaximize in TForm(Owner).BorderIcons then
    FBtnSet := FBtnSet + [btnMaximize]
  else
  begin
    FBtnSet := FBtnSet - [btnMaximize];
    TForm(Owner).BorderStyle := bsSingle;
  end;

  if not (biSystemMenu in TForm(Owner).BorderIcons) then
    FBtnSet := []
  else
    FBtnSet := FBtnSet + [btnClose];

  case TForm(Owner).BorderStyle of
    bsDialog: FBtnSet := [btnClose, btnMinimize];
  end;
//
  FCapButton := TBitmap.Create;
//
  FMacCapStyle := mcsMacOSLine;
  FActCapColor := clCaptionText;
  FInactCapColor := clInactiveCaptionText;
  FActFontColor := clActiveCaption;
  FInactFontColor := clInactiveCaption;
  FLDColor := clWhite;
  FLDShadow := clBtnShadow;
  FBorderColor := clBtnShadow;
//
  FCapFont := TFont.Create;
  FCapFont.Name := DefaultFont;
  FCapFont.Size := DefaultFontSize;
  FCapFont.Style := DefaultFontStyle;
  FPaintBorder := True;
//
  CY_CAPTION := GetSystemMetrics(SM_CYCAPTION);
  CX_ICON := GetSystemMetrics(SM_CXICON);
  CY_ICON := GetSystemMetrics(SM_CYICON);
  CX_EDGE := GetSystemMetrics(SM_CXEDGE);
  CY_EDGE := GetSystemMetrics(SM_CYEDGE);
//
end;
{******************************************************************************}
{       TodCustomMacOSCaption.Destroy:                                         }
{               Component destructor                                           }
{******************************************************************************}
destructor TodCustomMacOSCaption.Destroy;
begin
  FCapFont.Free;
{  if Assigned(ODtlbMain) then
  begin
//    TForm(Owner).Menu := ODtlbMain.Menu;
    ODtlbMain.Menu := nil;
    try
      FreeAndNil(ODtlbMain);
    except
      ODtlbMain := nil
    end;
  end;   }
  if Assigned(FCapButton) then FCapButton.Free;
  if Assigned(FCaptionBmp) then FCaptionBmp.Free;
//  FreeParentWindowHook;
  inherited Destroy;
end;
{******************************************************************************}
{                       Paint form border                                      } 
{******************************************************************************}
{       TodCustomMacOSCaption.PaintBorder:                                     }
{               Paint form border                                              }
{******************************************************************************}
procedure TodCustomMacOSCaption.PaintBorder;
var
  dc: hDc;
  Pen: hPen;
  OldPen: hPen;
  OldBrush: hBrush;
begin
  dc := GetWindowDC(TForm(Owner).Handle);
  Pen := CreatePen(PS_SOLID, 1, FBorderColor);
  OldPen := SelectObject(dc, Pen);
  OldBrush := SelectObject(dc, GetStockObject(NULL_BRUSH));
  Rectangle(dc, 0, 0, TForm(Owner).Width, TForm(Owner).Height);
  SelectObject(dc, OldBrush);
  SelectObject(dc, OldPen);
  DeleteObject(Pen);
  ReleaseDC(TForm(Owner).Handle, dc);
end;

{******************************************************************************}
{                       Creating and painting form caption                     } 
{******************************************************************************}
{       TodCustomMacOSCaption.DrawIcon:                                        }
{               Draw form icon in form caption                                 }
{******************************************************************************}
procedure TodCustomMacOSCaption.DrawIcon(DC: HDC; R: TRect);
var
  IconHandle		: HIcon;
  IconDC		: HDC;
  OldIconBMP, HIconBMP  : HBitmap;
  Brush, OldBrush	: HBrush;
begin
  with TForm(Owner) do
  begin
    if Icon.Handle <> 0 then
      IconHandle := Icon.Handle
    else
      if Application.Icon.Handle <> 0 then
	IconHandle := Application.Icon.Handle
      else
	IconHandle := LoadIcon(0, IDI_APPLICATION);
  end;
  IconDC := CreateCompatibleDC(DC);
  HIconBMP := CreateCompatibleBitmap(DC, CX_ICON, CY_ICON);
  OldIconBMP := SelectObject(IconDC, HIconBMP);
  Brush := CreateSolidBrush(ColorToRGB(clBtnFace));
  OldBrush := SelectObject(IconDC, Brush);
  PatBlt(IconDC, 0, 0, CX_ICON, CY_ICON, PATCOPY);

  Windows.DrawIcon(IconDC, 0, 0, IconHandle);
  StretchBlt(DC, R.Left, R.Top, R.Bottom-R.Top, R.Bottom-R.Top, IconDC,
	     0, 0, CX_ICON, CY_ICON, SRCCOPY);

  DeleteObject(SelectObject(IconDC, OldIconBMP));
  DeleteObject(SelectObject(IconDC, OldBrush));
  DeleteDC(IconDC);
end;
{******************************************************************************}
{       TodCustomMacOSCaption.CreateCaption:                                   }
{               Creating form caption bitmap                                   }
{******************************************************************************}
procedure TodCustomMacOSCaption.CreateCaption(Rec: TRect; Name: String; CActive: Boolean; var BMP: TBitMap);
var
  i,j: Integer;
  LeftC, RightC: Integer;
  TextBeg, TextEnd: Integer;
  newline: Boolean;
  FCaption: String;
  NextChar: Char;
begin
// Draw initial bitmap
  Bmp.Height := Rec.Bottom - Rec.Top;
  Bmp.Width := Rec.Right - Rec.Left;
  Bmp.Canvas.Pen.Color := clBtnFace;
  Bmp.Canvas.Brush.Style := bsSolid;
  Bmp.Canvas.Brush.Color := clBtnFace;
  Bmp.Canvas.FillRect(Rect(0, 0, Bmp.Width, Bmp.Height {- FMenuOffset}));
//
  FCaption := Name;
// Find left position for caption bitmap
  LeftC := _ButtonEdgeSpace + _FormEdgeSize + _ButtonSize;
// Draw form icon
  FIconRgn := Rect(LeftC + 2, 3, LeftC + 18, 19);
  DrawIcon(Bmp.Canvas.Handle, FIconRgn);
//
  Inc(LeftC, 16 + _BCSpace);
//
// Find right position for caption bitmap
  RightC := Bmp.Width + _FormEdgeSize - _ButtonEdgeSpace - _ButtonSize*2 -
            _ButtonSpace - _BCSpace;
//

// Include caption text in caption bitmap
  with Bmp do
  begin
//
    Canvas.Font.Name := FCapFont.Name;
    Canvas.Font.Style := FCapFont.Style;
    Canvas.Font.Size := FCapFont.Size;
//
    if CActive then
      Canvas.Font.Color := FActFontColor
    else
      Canvas.Font.Color := FInactFontColor;

    TextBeg := Round((RightC + LeftC - Canvas.TextWidth(Name) - _BCSpace*2)/2);

    if Length(Name) = 0 then
      TextEnd := TextBeg
    else
      TextEnd:= TextBeg + Canvas.TextWidth(Name) + _BCSpace*2;
  end;
//
  case FMacCapStyle of
    mcsMacOSLine:  // Line style caption
      begin
        i := _CaptionTop + 1;
        with BMP.Canvas do
        repeat
          Pen.Color := FLDColor;
    //  Draw left caption line
          if LeftC < TextBeg then
          begin
            MoveTo(LeftC, i);
            LineTo(TextBeg, i);
          end;
    //  Draw right caption line
          if RightC > TextEnd then
          begin
            MoveTo(TextEnd, i);
            LineTo(RightC, i);
          end;
          inc(i, 1);
          Pen.Color := FLDShadow;
    //  Draw left shadow caption line
          if LeftC < TextBeg then
          begin
            MoveTo(LeftC, i);
            LineTo(TextBeg, i);
          end;
    //  Draw right shadow caption line
          if RightC > TextEnd then
          begin
            MoveTo(TextEnd, i);
            LineTo(RightC, i);
          end;
          inc(i);
        until i >= Bmp.Height - 4 {- FMenuOffset};
      end;
    mcsMacOSDot:        // Dot style caption
      begin
        i := _CaptionTop + 1;
        newline := false;
        with BMP.Canvas do
          repeat
            if newline then
              j := 0
            else
              j := 2;
            repeat
              Pen.Color := LDColor;
      //  Draw left caption point
              if LeftC + j < TextBeg then
                Pixels[LeftC + j, i] := FLDColor;
      //  Draw right caption point
              if RightC - j > TextEnd then
                Pixels[RightC - j, i] := FLDColor;
      //  Draw left shadow caption point
              if LeftC + j < TextBeg then
                Pixels[LeftC + j, i + 1] := FLDShadow;
      //  Draw right shadow caption point
              if RightC - j > TextEnd then
                Pixels[RightC - j, i + 1] := FLDShadow;
              inc(j, 4);
             until j > Bmp.Width;
             newline := not newline;
             inc(i, 2)
           until i >= Bmp.Height - 4{ - FMenuOffset};
        end;
    mcsColor:
      begin
        Bmp.Canvas.Pen.Color := clBtnShadow;
        if CActive then
          Bmp.Canvas.Brush.Color := FActCapColor
        else
          Bmp.Canvas.Brush.Color := FInactCapColor;
        if LeftC < TextBeg then
        begin
          BMP.Canvas.Rectangle(Rect(LeftC, _FormEdgeSize + 2, TextBeg, BMP.Height {- FMenuOffset} - 2));
          BMP.Canvas.Rectangle(Rect(TextEnd, _FormEdgeSize + 2, RightC, BMP.Height {- FMenuOffset} - 2));
        end;
      end;
  end;
//
  BMP.Canvas.Brush.Color := clBtnFace;
  BMP.Canvas.Brush.Style := bsClear;
//
  if LeftC > TextBeg then
    TextBeg := LeftC;
//
  if (RightC - TextBeg) <= (BMP.Canvas.TextWidth(FCaption)) then
  begin
    FCaption := '';
    i := 1;
    repeat
      FCaption := FCaption + Name[i];
      inc(i);
      if i < Length(Name) then
        NextChar := Name[i];
    until (i >= Length(Name)) or
          ((RightC - TextBeg) < (BMP.Canvas.TextWidth(FCaption + NextChar + '...') + _BCSpace*2));
    FCaption := FCaption + '...';
  end;
  BMP.Canvas.TextRect(Rect(TextBeg, _CaptionFontTop, RightC, CY_CAPTION - 2),
           TextBeg + _BCSpace, _CaptionFontTop, FCaption);
//
  BMP.Canvas.Brush.Color := clBtnFace;
  BMP.Canvas.Brush.Style := bsSolid;
//
  Bmp.Canvas.FillRect(Rect(0, CY_CAPTION, Bmp.Width, CY_CAPTION {+ FMenuOffset}));
end;
{******************************************************************************}
{       TodCustomMacOSCaption.DrawCap:                                         }
{               Draw form caption bitmap                                       }
{******************************************************************************}
procedure TodCustomMacOSCaption.DrawCap(AActive: Boolean; ClBtn, MaxBtn, MinBtn: String);
var
  R1, R2: TRect;
  WindRect: TRect;
  HDL: THandle;
begin
  HDL := TForm(Owner).Handle;
//
{  if GetMenu(HDL) <> 0 then
    FMenuOffset := 20
  else
    FMenuOffset := 0;      }
//
  TForm(Owner).canvas.Handle := GetWindowDC(HDL);
//  
  GetWindowRect(HDL, WindRect);
  WindRect.Bottom := WindRect.Bottom - WindRect.Top;;
  WindRect.Right := WindRect.Right - WindRect.Left;
  WindRect.Left := 0;
  WindRect.Top := 0;
//
  TForm(Owner).Canvas.Pen.Color := TForm(Owner).Color;
  TForm(Owner).Canvas.Pen.Style := psInsideFrame;
  TForm(Owner).Canvas.Pen.Width := 4;
  TForm(Owner).Canvas.Brush.Style := bsClear;
  TForm(Owner).Canvas.Rectangle(0, 0, TForm(Owner).Width, TForm(Owner).Height);

  DrawEdge(TForm(Owner).canvas.Handle, WindRect, EDGE_RAISED, BF_RECT);

  r1.Left := CX_EDGE;
  r1.Top := CY_EDGE;
  r1.Right := WindRect.Right - CX_EDGE*2;
  r1.Bottom := CY_CAPTION +
               2*CY_EDGE{ + FMenuOffset};
  R2 := R1;
//
  CreateCaption(R1, TForm(Owner).Caption, AActive, FCaptionBmp);
  FillRect(TForm(Owner).Canvas.Handle, R1, CreatePatternBrush(FCaptionBmp.Handle));

//  SendMessage(TForm(Owner).Handle, WM_MENUCHAR, 0, 0);

  if btnClose in FBtnSet then
    FCapButton.LoadFromResourceName(HInstance, PChar('MACCLOSE' + ClBtn))
  else
    FCapButton.LoadFromResourceName(HInstance, PChar('MACCLOSED'));
  TForm(Owner).Canvas.Draw(r2.Left + _ButtonEdgeSpace,
                           r2.Top + _ButtonEdgeSpace,
                           FCapButton);
  FCloseRgn := Rect(r2.Left + _ButtonEdgeSpace,
                    r2.Top + _ButtonEdgeSpace,
                    r2.Left + _ButtonEdgeSpace + _ButtonSize,
                    r2.Top + _ButtonEdgeSpace + _ButtonSize);

  if btnMinimize in FBtnSet then
    FCapButton.LoadFromResourceName(HInstance,PChar('MACMIN' + MinBtn))
  else
    FCapButton.LoadFromResourceName(HInstance,PChar('MACMIND'));
  TForm(Owner).Canvas.Draw(r2.Right - _ButtonEdgeSpace - _ButtonSize,
                           r2.Top + _ButtonEdgeSpace, FCapButton);
  FMinRgn := Rect(r2.Right - _ButtonEdgeSpace - _ButtonSize,
                  r2.Top + _ButtonEdgeSpace,
                  r2.Right - _ButtonEdgeSpace,
                  r2.Top + _ButtonEdgeSpace + _ButtonSize);

  if btnMaximize in FBtnSet then
    FCapButton.LoadFromResourceName(HInstance,PChar('MACMAX' + MaxBtn))
  else
    FCapButton.LoadFromResourceName(HInstance,PChar('MACMAXD'));
  TForm(Owner).Canvas.Draw(r2.Right - _ButtonEdgeSpace - _ButtonSpace - _ButtonSize*2,
                           r2.Top + _ButtonEdgeSpace, FCapButton);
  FMaxRgn := Rect(r2.Right - _ButtonEdgeSpace - _ButtonSpace - _ButtonSize*2,
                  r2.Top + _ButtonSpace,
                  r2.Right - _ButtonEdgeSpace - _ButtonSpace - _ButtonSize,
                  r2.Top + _ButtonEdgeSpace + _ButtonSize);

//
  if FPaintBorder then
    PaintBorder;
//
  ReleaseDC(HDL, TForm(Owner).Canvas.Handle);
  TForm(Owner).Canvas.Handle := 0;
//
end;

end.
